__version__ = '0.0.0'

__author_name__ = 'author name'
__author_email__ = 'author@email.com'
__url_version__ = '/v' + __version__.split('.')[0]

__author__ = f"{__author_name__} <{__author_email__}>"

__all__ = [
    '__version__',
    '__author__',
    '__author_name__',
    '__author_email__',
    '__url_version__',
]
